# Описание проекта

Доработка автоматизированной системы планирования и выполнения численного моделирования процессов сжигания топлив в горелочных устройствах энергоустановок предназначена для оптимизации процессов проведения численных экспериментов с цифровой моделью горелочного устройства с целью поиска наиболее экономичных и низко-эмиссионных режимов работы

# Участники
1. Кашин Максим
2. Клюшенкова Ксения
3. Базунов Андрей
4. Жимолостнова Анна
5. Цуканова Ирина

# Запуск проекта
## 1. Создание окружения
```
py -m venv .venv
```
## 2. Переход в окружение
```
.\.venv\Scripts\activate 
```
## 3. Скачивание библиотек
```
pip install -r .\requirements.txt
```
## 4. Создаем .env
Необходимо создать файл и поместить в него необходимые параметры.
```
DATABASE=SuperService
POSTGRES_USER=UserSuperService
POSTGRES_PASSWORD=NotWarningWord1
CLICKHOUSE_USER=UserMyHouse
CLICKHOUSE_PASSWORD=NotWarningWord2
```

## 5. Запускаем все контейнеры
```
docker-compose up --build
```
При необходимости можем закрыть контейнера
```
docker-compose down
```

## 6. Запускаем проект
```
python runner.py  
```

## 7. Подключение к ClickHouse
Команда входа в ClickHouse
```
docker exec -it clickhouse-db clickhouse-client -u UserMyHouse --password NotWarningWord2 --host localhost
```
Использовать базу данных
```
USE SuperService;
```
## 8. Подключение к PostgreSQL
Команда входа в ClickHouse
```
docker exec -it postgres-db psql -U UserSuperService -d SuperService
```

## 9. Миграция alembic
```
alembic revision --autogenerate
```
```
alembic upgrade head
```

# Инициализация БД

## 1. Запустить docker-compose
```
docker-compose up --build
```
## 2. Зайти в ClickHouse
```
docker exec -it clickhouse-db clickhouse-client -u UserMyHouse --password NotWarningWord2 --host localhost
```
## 3. Создать базу данных
```
CREATE DATABASE SuperService;
```
## 4. Инициализировать БД
Зайти на fastapi и выполнить запрос:
```
http://localhost:8000/init_db_data
```
PostgreSQL и ClickHouse будут заполнены данными.