from fastapi import APIRouter, HTTPException

from db.crud import *
from db.models import ExperimentCategory
from network.schemas import ExperimentCategoryBody

router = APIRouter()



@router.post('/create')
async def create_experiment_category(body: ExperimentCategoryBody):
    try:
        await create(ExperimentCategory,
            name=body.name
        )

        return {"message": "Новая запись <ExperimentCategory> успешно добавлена"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"An error occurred: {str(e)}")


@router.get('/all')
async def get_all_experiment_category():
    try:
        result = await get_all(ExperimentCategory)

        if result is not None:
            return result
        else:
            return {"message": "Нет записей в <ExperimentCategory>"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"An error occurred: {str(e)}")


@router.get('/{id}')
async def get_by_id_experiment_category(id: int):
    try:
        result = await get_by_id(ExperimentCategory, id)

        if result is not None:
            return result
        else:
            return {"message": "Запись <ExperimentCategory> не найдена"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"An error occurred: {str(e)}")


@router.delete('/{id}/delete')
async def delete_experiment_category(id: int):
    try:
        is_deleted = await delete(ExperimentCategory, id)
        if is_deleted:
            return {"message": "Запись <ExperimentCategory> успешно удалена"}
        else:
            return {"message": "Запись <ExperimentCategory> не найдена"}

    except Exception as e:
        raise HTTPException(status_code=500, detail=f"An error occurred: {str(e)}")
